<?php
/**
* <b>DBWrapper</b> class.
* Abstraction class for database functionality
*/

Class DBWrapper
{
	public $conn;
	public $dbName;
	public $result;

	// -------------------------------------------------------------
	public function __construct($info = null)
	{
		$dbInfo = ($info != null) ? $info : $_SESSION['config']['db'];
		$this->dbName = $dbInfo['database'];
		$hostParts = explode(":", $dbInfo['host']);
		$serverName = $hostParts[0];
		$dbUser = $dbInfo['user'];
		$dbPassword = $dbInfo['pass'];
		$dbPort = ($hostParts[1] != "") ? $hostParts[1] : 3306;
		$this->conn = mysqli_connect($serverName, $dbUser, $dbPassword, $this->dbName, $dbPort);
		if (!$this->conn)
		{
			throw new Exception('Could not connect to the database. Either the database is down or connection parameters are wrong.'.$_SESSION['config']['db']['database']);
		}
	}

	// -------------------------------------------------------------
	public function __destruct()
	{
		if (!$this->conn)
		{
			mysqli_close($this->conn);
		}
	}

	// -------------------------------------------------------------
	public function Close()
	{
		mysqli_close($this->conn);
	}

	// -------------------------------------------------------------
	public function GetConnection() {
		return $this->conn;
	}

	// -------------------------------------------------------------
	public function Query($query)
	{
		//$_SESSION['logger']->log(LOG_VERB, $query);
		$this->result = mysqli_query($this->conn, $query);
		if (!$this->result) {
			throw new Exception(mysqli_errno($this->conn).":".mysqli_error($this->conn));
		}
		return $this->result;
	}

	// -------------------------------------------------------------
	public function NumRows()
	{
		if ($this->result != false)
		{
			return mysqli_num_rows($this->result);
		}
		return null;
	}

	// -------------------------------------------------------------
	public function AffectedRows()
	{
		return mysqli_affected_rows($this->conn);
	}

	// -------------------------------------------------------------
	public function Result($idBased = false)
	{
		if ($this->NumRows() > 0)
		{
			while($item = mysqli_fetch_assoc($this->result))
			{
				if ($idBased)
				{
					$results[$item["id"]] = $item;
				}
				else
				{
					$results[] = $item;
				}
			}
			return $results;
		}
		return null;
	}

	// -------------------------------------------------------------
	public function UpdateQuery($query)
	{
		//$_SESSION['logger']->log(LOG_VERB, $query);
		$this->result = mysqli_query($this->conn, $query) or die (mysqli_error($this->conn));
		return ($this->AffectedRows() > 0);
	}

	// -------------------------------------------------------------
	function Escape($text)
	{
		return mysqli_escape_string($this->conn, $text);
	}

	// -------------------------------------------------------------
	function Unescape($text)
	{
		return stripcslashes($text);
	}

	// -------------------------------------------------------------
	function GetCurrentId()
	{
		return intval(mysqli_insert_id($this->conn));
	}

	function GetFieldInfo($tableName)
	{
		$query = "SHOW COLUMNS FROM $tableName";
		$this->Query($query);
		$result = $this->Result();
		$tableProperties = array();
		foreach($result as $row)
		{
			// We will get Field, Type, Null, Key, Default & Extra
			// What we are interested in, is the maxLengh and field type
			$type = $row['Type'];
			$parenthesisStart = strpos($type, '(');
			$fieldType = substr($type, 0,  $parenthesisStart);
			$length = substr($type, $parenthesisStart+1, strpos($type, ')', $parenthesisStart));
			$row['FieldType'] = $fieldType;
			$row['Length'] = $length;
			$tableProperties[$row['Field']] = $row;
		}
		return $tableProperties;
	}
}
?>
