<?php
// Include configuration & DB Wrapper Class
include_once("config.inc.php");
include_once("DBWrapper.php");

// Connect to DB
$db = new DBWrapper($config['db']);
$table = $config['db']['table_paypal_history'];

// Create Masters for Customers, Products and Paypal
$query = "SELECT
CASE WHEN Net < 0
	THEN ToEmailAddress
	ELSE FromEmailAddress
	END AS EmailAddress,
CASE WHEN Net <0
	THEN 'Purchase Accounts'
	ELSE 'Sundry Debtors'
	END AS ParentLedger, Name
FROM `{$table}`
WHERE STATUS = 'Completed'
	AND (TYPE LIKE '%Received' OR TYPE LIKE '%Sent' )

UNION

SELECT ItemTitle AS EmailAddress, 'Sales Accounts' AS ParentLedger, ItemTitle AS Name
FROM `{$table}`
WHERE (TYPE LIKE '%Received' OR TYPE LIKE '%Item' )
	AND ToEmailAddress IN ( '".implode("', '", $config['paypal']['emails'])."' )
	AND ItemTitle != 'Shopping Cart'
	AND ItemTitle != ''
GROUP BY EmailAddress
ORDER BY ParentLedger, EmailAddress";

$db->Query($query);
$rows = $db->Result();

$ledgerMasters[] = '<ENVELOPE>
<HEADER>
<TALLYREQUEST>Import Data</TALLYREQUEST>
</HEADER>
<BODY>
<IMPORTDATA>
<REQUESTDESC>
<REPORTNAME>All Masters</REPORTNAME>
<STATICVARIABLES>
<SVCURRENTCOMPANY>'.$config['tally']['companyName'].'</SVCURRENTCOMPANY>
</STATICVARIABLES>
</REQUESTDESC>
<REQUESTDATA>';

$lastParent = '';
$sortPosition = 1001;

echo "Processing ".$db->NumRows()." masters...";

// Add some default Ledgers, Just in case they don't exist yet
$rows[] = array('EmailAddress' => 'Paypal', 'Name'=> 'Paypal', 'ParentLedger' => 'Bank Accounts');
$rows[] = array('EmailAddress' => 'Paypal Transaction Charges', 'Name'=> 'Paypal Transaction Charges', 'ParentLedger' => 'Direct Expenses');

foreach ($rows as $index => $item) {
	if ($lastParent != $item['ParentLedger'])
	{
		$lastParent = $item['ParentLedger'];
		$sortPosition = 1001;
	}
	$sortPosition++;
	$ledgerMasters[] = "<TALLYMESSAGE xmlns:UDF=\"TallyUDF\">
 <LEDGER NAME=\"{$item['EmailAddress']}\" RESERVEDNAME=\"\">
  <NAME.LIST>
    <NAME>{$item['EmailAddress']}</NAME>
  </NAME.LIST>
  <ADDITIONALNAME>{$item['Name']}</ADDITIONALNAME>
  <ISINTERESTON>No</ISINTERESTON>
  <PARENT>{$item['ParentLedger']}</PARENT>
  <CURRENCYNAME>Rs.</CURRENCYNAME>
  <ISBILLWISEON>No</ISBILLWISEON>
  <ISCOSTCENTRESON>Yes</ISCOSTCENTRESON>
  <AFFECTSSTOCK>No</AFFECTSSTOCK>
  <ISCONDENSED>No</ISCONDENSED>
  <SORTPOSITION> {$sortPosition}</SORTPOSITION>
  <INTERESTONBILLWISE>No</INTERESTONBILLWISE>
  <OVERRIDEINTEREST>No</OVERRIDEINTEREST>
  <OVERRIDEADVINTEREST>No</OVERRIDEADVINTEREST>
  <FORPAYROLL>No</FORPAYROLL>
  <PAYTYPE/>
  <SHOWINPAYSLIP>No</SHOWINPAYSLIP>
  <ASSLABRATE>No</ASSLABRATE>
  <TDSDEDUCTEEISSPECIALRATE>No</TDSDEDUCTEEISSPECIALRATE>
  <TDSDEDUCTEEISCACERTIFIED>No</TDSDEDUCTEEISCACERTIFIED>
  <ISTDSAPPLICABLE>No</ISTDSAPPLICABLE>
  <USEFORVAT>No</USEFORVAT>
  <IGNORETDSEXEMPT>No</IGNORETDSEXEMPT>
  <TDSDEDUCTEETYPE/>
  <TAXTYPE/>
  <TAXCLASSIFICATIONNAME/>
  <TDSRATENAME/>
 </LEDGER>
</TALLYMESSAGE>";
}

$ledgerMasters[] = '</REQUESTDATA>
</IMPORTDATA>
</BODY>
</ENVELOPE>';

file_put_contents("data/Master.xml", stripInvalid(implode("\r\n", $ledgerMasters)));
echo "Masters done.";
unset($ledgerMasters);
unset($rows);



// Now for the vouchers
$vouchers[] = '<ENVELOPE>
<HEADER>
<TALLYREQUEST>Import Data</TALLYREQUEST>
</HEADER>
<BODY>
<IMPORTDATA>
<REQUESTDESC>
<REPORTNAME>All Masters</REPORTNAME>
<STATICVARIABLES>
<SVCURRENTCOMPANY>'.$config['tally']['companyName'].'</SVCURRENTCOMPANY>
</STATICVARIABLES>
</REQUESTDESC>
<REQUESTDATA>';

//Sales & Receipt
//Receipt: Paypal Dr 195, Paypal Charges Dr 5, Mr. X Cr 200
//Sales: Mr. X Dr, Product Cr. (100) 

$query = "SELECT 
CASE WHEN (Type LIKE '%Sent' ";
foreach($config['paypal']['emails'] as $email)
	$query .= " OR FromEmailAddress = '{$email}' ";
$query .= ") THEN 'Purchase'
	ELSE 'Sales'
	END AS VoucherType,
Date, Time, Name, Type as TransactionType, Currency, Gross, Fee, Net, FromEmailAddress, ToEmailAddress, TransactionID, ItemTitle, InvoiceNumber, Quantity, ReceiptID
FROM `{$table}`
WHERE Status = 'Completed' AND (Type LIKE '%Received' OR Type LIKE '%Sent'  OR Type LIKE '%Item' )
ORDER BY `Date`, `Time`, TransactionID, Type";

$db->Query($query);
$rows = $db->Result();

$lastParent = '';
$voucherID = $config['tally']['startingVoucherID'];
$cartItems = array();

echo "Processing ".$db->NumRows()." vouchers...";

foreach ($rows as $index => $item) {

	$item['Date'] = str_replace('-', '', $item['Date']);
	
	$vStart = "<TALLYMESSAGE xmlns:UDF=\"TallyUDF\">
				 <VOUCHER REMOTEID=\"30647ae3-f9e2-4655-8583-fd577fca821c-{VoucherID}\" VCHTYPE=\"{VoucherType}\" ACTION=\"Create\">
				  <ISOPTIONAL>No</ISOPTIONAL>
				  <USEFORGAINLOSS>No</USEFORGAINLOSS>
				  <USEFORCOMPOUND>No</USEFORCOMPOUND>
				  <VOUCHERTYPENAME>{VoucherType}</VOUCHERTYPENAME>
				  <DATE>{$item['Date']}</DATE>
				  <EFFECTIVEDATE>{$item['Date']}</EFFECTIVEDATE>
				  <ISCANCELLED>No</ISCANCELLED>
				  <USETRACKINGNUMBER>No</USETRACKINGNUMBER>
				  <ISPOSTDATED>No</ISPOSTDATED>
				  <ISINVOICE>No</ISINVOICE>
				  <DIFFACTUALQTY>No</DIFFACTUALQTY>
				  <VOUCHERNUMBER>{VoucherID}</VOUCHERNUMBER>
				  <ASPAYSLIP>No</ASPAYSLIP>
				  <GUID>30647ae3-f9e2-4655-8583-fd577fca821c-{VoucherID}</GUID>
				  <ALTERID> {VoucherID}</ALTERID>";
	
	$vEnd = "</VOUCHER></TALLYMESSAGE>";
	
	switch($item['VoucherType'])
	{
		case 'Purchase':
			continue;
			break;
		case 'Sales':
			if ($item['TransactionType'] == 'Shopping Cart Item')
			{
				$cartItems[] = array('ItemTitle'=> $item['ItemTitle'], 'Gross' => $item['Gross']);
				continue;
				break;
			}
			if ($item['TransactionType'] != 'Web Accept Payment Received' && $item['TransactionType'] != 'Shopping Cart Payment Received')
			{
				$cartItems = array();
				continue;
				break;	
			}
			// Entry for Sales
			$item['AdjustedAmount'] = convertCurrency( $item['Gross'], $item['Date'], $item['Currency']);
			
			// Single item sales
			if ($item['TransactionType'] == 'Web Accept Payment Received')
			{
				$vP = "<ALLLEDGERENTRIES.LIST>
					   <REMOVEZEROENTRIES>No</REMOVEZEROENTRIES>
					   <ISDEEMEDPOSITIVE>No</ISDEEMEDPOSITIVE>
					   <LEDGERFROMITEM>No</LEDGERFROMITEM>
					   <LEDGERNAME>{$item['ItemTitle']}</LEDGERNAME>
					   <AMOUNT>{$item['AdjustedAmount']}</AMOUNT>
					  </ALLLEDGERENTRIES.LIST>";
			}
			// Multiple item sales
			else if ($item['TransactionType'] == 'Shopping Cart Payment Received')
			{
				$cartItemNames = array();
				$vP = '';
				$item['AdjustedAmount'] = 0;
				foreach($cartItems as $cartItem)
				{
					$cartItem['AdjustedAmount'] = convertCurrency($cartItem['Gross'], $item['Date'], $item['Currency']);
					$item['AdjustedAmount'] += $cartItem['AdjustedAmount'];
					$vP .= "<ALLLEDGERENTRIES.LIST>
						   <REMOVEZEROENTRIES>No</REMOVEZEROENTRIES>
						   <ISDEEMEDPOSITIVE>No</ISDEEMEDPOSITIVE>
						   <LEDGERFROMITEM>No</LEDGERFROMITEM>
						   <LEDGERNAME>{$cartItem['ItemTitle']}</LEDGERNAME>
						   <AMOUNT>{$cartItem['AdjustedAmount']}</AMOUNT>
						  </ALLLEDGERENTRIES.LIST>";
					$cartItemNames[] = $cartItem['ItemTitle'];
				}
				$item['ItemTitle'] = implode(", ", $cartItemNames);
			}
			
			
			$v = "<REFERENCE>{$item['TransactionID']}</REFERENCE>";  
			$v .= "<ALLLEDGERENTRIES.LIST>
				   <REMOVEZEROENTRIES>No</REMOVEZEROENTRIES>
				   <ISDEEMEDPOSITIVE>Yes</ISDEEMEDPOSITIVE>
				   <LEDGERFROMITEM>Yes</LEDGERFROMITEM>
				   <LEDGERNAME>{$item['FromEmailAddress']}</LEDGERNAME>
				   <AMOUNT>-{$item['AdjustedAmount']}</AMOUNT>
				  </ALLLEDGERENTRIES.LIST> ";
			$v .= $vP;
			$v .= "<NARRATION>Being sales of {$item['ItemTitle']}. {$item['Currency']} @ ".convertCurrency(1, $item['Date'], $item['Currency']).". Invoice: {$item['InvoiceNumber']} ";
			if ($item['ReceiptID'] != '') $v .= "Receipt: {$item['ReceiptID']}";
			$v .= "</NARRATION>";					
			$vouchers[] = str_replace('{VoucherID}', $voucherID, str_replace('{VoucherType}', 'Sales', $vStart)) . "\r\n" . $v . "\r\n" . $vEnd;
			$voucherID++;
			
			
			// Entry for Receipt
			$v = "<REFERENCE>{$item['TransactionID']}</REFERENCE>";
			$v .= "<NARRATION>Being payment received for {$item['ItemTitle']}. Invoice: {$item['InvoiceNumber']} ";
			if ($item['ReceiptID'] != '') $v .= "Receipt: {$item['ReceiptID']}";
			$v .= "</NARRATION>";

			$net = convertCurrency( $item['Net'], $item['Date'], $item['Currency']);
			$fee = convertCurrency( abs($item['Fee']), $item['Date'], $item['Currency']);
			$gross = $net + $fee;
			$v .= "<ALLLEDGERENTRIES.LIST>
				   <REMOVEZEROENTRIES>No</REMOVEZEROENTRIES>
				   <ISDEEMEDPOSITIVE>No</ISDEEMEDPOSITIVE>
				   <LEDGERFROMITEM>No</LEDGERFROMITEM>
				   <LEDGERNAME>{$item['FromEmailAddress']}</LEDGERNAME>
				   <AMOUNT>{$gross}</AMOUNT>
				  </ALLLEDGERENTRIES.LIST>";
     
			$v .= "<ALLLEDGERENTRIES.LIST>
				   <REMOVEZEROENTRIES>No</REMOVEZEROENTRIES>
				   <ISDEEMEDPOSITIVE>Yes</ISDEEMEDPOSITIVE>
				   <LEDGERFROMITEM>Yes</LEDGERFROMITEM>
				   <LEDGERNAME>Paypal</LEDGERNAME>
				   <AMOUNT>-{$net}</AMOUNT>
				  </ALLLEDGERENTRIES.LIST>";
			
			if ($item['Fee'] != 0)
			{
				$v .= "<ALLLEDGERENTRIES.LIST>
					   <REMOVEZEROENTRIES>No</REMOVEZEROENTRIES>
					   <ISDEEMEDPOSITIVE>Yes</ISDEEMEDPOSITIVE>
					   <LEDGERFROMITEM>Yes</LEDGERFROMITEM>
					   <LEDGERNAME>Paypal Transaction Charges</LEDGERNAME>
					   <AMOUNT>-{$fee}</AMOUNT>
					  </ALLLEDGERENTRIES.LIST>";
			}
			$vouchers[] = str_replace('{VoucherID}', $voucherID, str_replace('{VoucherType}', 'Receipt', $vStart)) . "\r\n" . $v . "\r\n" . $vEnd;
			$voucherID++;
			$cartItems = array();
			break;
	}
}
$vouchers[] = '</REQUESTDATA></IMPORTDATA></BODY></ENVELOPE>';
file_put_contents("data/DayBook.xml", stripInvalid(implode("\r\n", $vouchers)));
echo "Vouchers done.";
unset($vouchers);
unset($rows);
?>